package HT.fx;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Scanner;

import fi.jyu.mit.ohj2.Mjonot;

/**
 * @author joonatan, tuomas
 * @version 13.3.2024
 *
 */
public class Pelaajat {
    private static final int Max_Pelaajat   = 9;
    private int lkm           = 0;
    private String tiedostonNimi = "";
    private Pelaaja alkiot[]      = new Pelaaja[Max_Pelaajat];
    
    /**
     * Oletusmuodostaja
     */
    public Pelaajat() {
        // Attribuuttien oma alustus riittää
    }
    
    /**
     * @param pelaaja pelaaja
     * @throws SailoException exception kun rakenne täysi
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * Pelaajat pelaajat = new Pelaajat();
     * Pelaaja aku1 = new Pelaaja(), aku2 = new Pelaaja();
     * pelaajat.getLkm() === 0;
     * pelaajat.lisaa(aku1); pelaajat.getLkm() === 1;
     * pelaajat.lisaa(aku2); pelaajat.getLkm() === 2;
     * pelaajat.lisaa(aku1); pelaajat.getLkm() === 3;
     * pelaajat.anna(0) === aku1;
     * pelaajat.anna(1) === aku2;
     * pelaajat.anna(2) === aku1;
     * pelaajat.anna(1) == aku1 === false;
     * pelaajat.anna(1) == aku2 === true;
     * pelaajat.anna(3) === aku1; #THROWS IndexOutOfBoundsException 
     * pelaajat.lisaa(aku1); pelaajat.getLkm() === 4;
     * pelaajat.lisaa(aku1); pelaajat.getLkm() === 5;
     * pelaajat.lisaa(aku1);  #THROWS SailoException
     * </pre>
     */
    public void lisaa(Pelaaja pelaaja) throws SailoException {
        if (lkm >= alkiot.length) lisaatilaa();
        alkiot[lkm] = pelaaja;
        lkm++;
    }
    
    private void lisaatilaa() {
        Pelaaja alkiot2[] = new Pelaaja[lkm + 1];
        int i = 0;
        while(i<lkm) {
            alkiot2[i] = alkiot[i];
            i++;
        }
        alkiot = alkiot2;
        
    }

    /**
     * Palauttaa viitteen iinteen jäseneen.
     * @param i monennenko jäsenen viite halutaan
     * @return viite jäseneen, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella  
     */
    public Pelaaja anna(int i) throws IndexOutOfBoundsException {
        if (i < 0 || lkm < i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        
        Pelaaja palautettava = null;
        int m = 0;
        while(m<alkiot.length) {
            if(alkiot[m]!=null) {
                if(alkiot[m].getTunnusNro() == i) {
                    palautettava = alkiot[m]; 
                    break;
                    }
            }
            m++;
            }
            
        
        return palautettava;
    }
    
    
    /**
     * Lukee pelaajiston tiedostosta.  Ei toimi.
     * @param tiedosto tiedoston hakemisto
     * @throws SailoException jos lukeminen epäonnistuu
     */
    public void lueTiedostosta(String tiedosto) throws SailoException {
        
        //tiedostonNimi = tiedosto;
        try (Scanner fi = new Scanner(new FileInputStream(new File(tiedosto)))) { // Jotta UTF8/ISO-8859 toimii

            
            while ( fi.hasNext() ) {
                String s = fi.nextLine();
                if(s.length()>1) {
                    
                    
                    StringBuffer sb = new StringBuffer(s);
                    Pelaaja pel = new Pelaaja();
                    pel.parse(sb);
                    lisaa(pel);
                   
                    
                }
            }
        } catch (FileNotFoundException ex) {
            System.err.println("Tiedosto ei aukea! " + ex.getMessage());
            ex.printStackTrace();
            return;
        }
        
        
    }
    
    /**
     * Tallentaa pelaajiston tiedostoon.  Ei toimi.
     * @throws SailoException jos talletus epäonnistuu
     */
    public void talleta() throws SailoException {
        try (PrintStream fo = new PrintStream(new FileOutputStream("Pelaajatiedot", false)))  {
            int i = 1;
            while ( i<=PadelMainGUIController.getKarkelo().getPelaajia()) {
                if(PadelMainGUIController.getKarkelo().annaPelaaja(i)!=null) {
                    fo.print(PadelMainGUIController.getKarkelo().annaPelaaja(i).getTiedot()+ "\n");
                    
                }
                i++;
            }
            //System.out.println("Heureka ! ");
        } catch(FileNotFoundException ex) {
            System.err.println("Tiedosto ei aukea: " + ex.getMessage());
        }
    }
    
    /**
     * Palauttaa jäsenten lukumäärän
     * @return jäsenten lukumäärä
     */
    public int getLkm() {
        return lkm;
    }
    
    /**
     * Testiohjelma jäsenistölle
     * @param args ei käytössä
     */
    public static void main(String args[]) {
        Pelaajat pelaajat = new Pelaajat();

      Pelaaja aku = new Pelaaja(), aku2 = new Pelaaja();
        aku.rekisteroi();
        aku.luoTesti();
        aku2.rekisteroi();
        aku2.luoTesti();

        try {
            pelaajat.lisaa(aku);
            pelaajat.lisaa(aku2);

            System.out.println("============= Jäsenet testi =================");

            for (int i = 0; i < pelaajat.getLkm(); i++) {
                Pelaaja pelaaja = pelaajat.anna(i);
                System.out.println("Pelaaja nro: " + i);
                pelaaja.tulosta(System.out);
            }

        } catch (SailoException ex) {
            System.out.println(ex.getMessage());
        }
    }

    /**
     * Käydään taulukko lä'pi ja luodaan uusi ilman poistettavaa pelaajaa
     * @param pel pelaaja joka poistetaan
     */
    public void poistaPelaaja(Pelaaja pel) {
        Pelaaja alkiot2[] = new Pelaaja[lkm-1];
        int i = 0;
        int n = 0;
        while(i<lkm) {
            if(alkiot[i].getTunnusNro() != pel.getTunnusNro()) {
                alkiot2[n] = alkiot[i];
                n++;
            }
            i++;
        }
        //lkm--;
        alkiot = alkiot2;
        
    }
}
